@@ -1,10 +1,14 @@
 import abc
 
 import six
+import websocket
 
-from .common import ListResources, GetResource, CreateResource, StartResource, StopResource, DeleteResource
-from .. import config, serializers
-from ..serializers import utils
+from .common import ListResources, CreateResource, StartResource, StopResource, DeleteResource, GetResource, GetMetrics, \
+    StreamMetrics, ListLogs
+from .. import config, serializers, sdk_exceptions
+from ..repositories.jobs import ListJobs
+from ..serializers import utils as serializers_utils
+from ..utils import concatenate_urls
 
 
 class GetBaseExperimentApiUrlMixin(object):
@@ -26,7 +30,7 @@ class ParseExperimentDictMixin(object):
         :param dict experiment_dict:
         :rtype BaseExperiment
         """
-        serializer = utils.get_serializer_for_experiment(experiment_dict)
+        serializer = serializers_utils.get_serializer_for_experiment(experiment_dict)
         experiment = serializer().get_instance(experiment_dict)
         return experiment
 
@@ -94,46 +98,10 @@ class GetExperiment(ParseExperimentDictMixin, GetBaseExperimentApiUrlMixin, GetR
         return url
 
 
-class ListExperimentLogs(ListResources):
-    def _get_api_url(self, **kwargs):
-        return config.config.CONFIG_LOG_HOST
-
-    def get_request_url(self, **kwargs):
-        return "/jobs/logs"
-
-    def list(self, experiment_id, line, limit, **kwargs):
-        instances = super(ListExperimentLogs, self).list(experiment_id=experiment_id,
-                                                         line=line, limit=limit)
-        instances = list(instances)  # because here the _parse_objects returns a generator
-        return instances
-
-    def yield_logs(self, experiment_id, line=0, limit=10000):
-
-        gen = self._get_logs_generator(experiment_id, line, limit)
-        return gen
-
-    def _get_logs_generator(self, experiment_id, line, limit):
-        last_line_number = line
-
-        while True:
-            logs = self.list(experiment_id, last_line_number, limit)
-
-            for log in logs:
-                # stop generator - "PSEOF" indicates there are no more logs
-                if log.message == "PSEOF":
-                    return
-
-                last_line_number += 1
-                yield log
-
-    def _parse_objects(self, log_rows, **kwargs):
-        serializer = serializers.LogRowSchema()
-        log_rows = (serializer.get_instance(row) for row in log_rows)
-        return log_rows
-
+class ListExperimentLogs(ListLogs):
     def _get_request_params(self, kwargs):
         params = {
-            "experimentId": kwargs["experiment_id"],
+            "experimentId": kwargs["id"],
             "line": kwargs["line"],
             "limit": kwargs["limit"],
         }
@@ -195,3 +163,63 @@ class DeleteExperiment(GetBaseExperimentApiUrlMixin, DeleteResource):
     def get_request_url(self, **kwargs):
         experiment_id = kwargs["id"]
         return "/experiments/{}/".format(experiment_id)
+
+
+class GetExperimentMetricsApiUrlMixin(object):
+    def _get_metrics_api_url(self, instance, protocol="https"):
+        instance_id = getattr(instance, "id", instance)
+        repository = ListJobs(api_key=self.api_key, logger=self.logger, ps_client_name=self.ps_client_name)
+        try:
+            job = repository.list(experiment_id=instance_id)[0]
+        except IndexError:
+            raise sdk_exceptions.GradientSdkError("Experiment has not started yet")
+
+        metrics_api_url = concatenate_urls(protocol + "://", job.metrics_url)
+        if not job.metrics_url:
+            raise sdk_exceptions.GradientSdkError("Metrics API url not found")
+
+        return metrics_api_url
+
+
+class GetExperimentMetrics(GetExperimentMetricsApiUrlMixin, GetMetrics):
+    OBJECT_TYPE = "experiment"
+
+    def _get_instance_by_id(self, instance_id, **kwargs):
+        repository = GetExperiment(self.api_key, logger=self.logger, ps_client_name=self.ps_client_name)
+        instance = repository.get(experiment_id=instance_id)
+        return instance
+
+    def _get_start_date(self, instance, kwargs):
+        rv = super(GetExperimentMetrics, self)._get_start_date(instance, kwargs)
+        if rv is None:
+            raise sdk_exceptions.GradientSdkError("Experiment has not started yet")
+
+        return rv
+
+    def _get_instance(self, response, **kwargs):
+        try:
+            rv = super(GetExperimentMetrics, self)._get_instance(response, **kwargs)
+        except sdk_exceptions.ResourceFetchingError as e:
+            if '{"version":' in str(e):
+                # TODO: metrics are not working for v1 experiments at the moment
+                raise sdk_exceptions.GradientSdkError("Metrics are available for private clusters only")
+            else:
+                raise
+
+        return rv
+
+
+class StreamExperimentMetrics(GetExperimentMetricsApiUrlMixin, StreamMetrics):
+    OBJECT_TYPE = "experiment"
+
+    def _get_connection(self, kwargs):
+        try:
+            ws = super(StreamExperimentMetrics, self)._get_connection(kwargs)
+        except websocket.WebSocketBadStatusException as e:
+            if "Handshake status 200 OK" in str(e):
+                # TODO: metrics are not working for v1 experiments at the moment
+                raise sdk_exceptions.GradientSdkError("Metrics are available for private clusters only")
+            else:
+                raise
+
+        return ws
