@@ -1,17 +1,15 @@
 import base64
+import os
 import random
 import string
 from collections import OrderedDict
 
+import progressbar
 import six
+from requests_toolbelt.multipart import encoder
 
 from . import constants, sdk_exceptions
 
-try:
-    from urlparse import urljoin as urljoin_original
-except ImportError:
-    from urllib.parse import urljoin as urljoin_original
-
 
 class MessageExtractor(object):
     def get_message_from_response_data(self, response_data, sep="\n"):
@@ -108,10 +106,81 @@ def base64_encode(s):
 
 
 def base64_encode_attribute(data, name):
-    encoded_value = base64_encode(getattr(data, name))
+    raw = getattr(data, name)
+    if raw is None:
+        return
+    encoded_value = base64_encode(raw)
     setattr(data, name, encoded_value)
 
 
-def urljoin(base, url):
-    url = urljoin_original(base, url)
-    return str(url)
+def concatenate_urls(fst_part, snd_part):
+    fst_part = fst_part if not fst_part.endswith("/") else fst_part[:-1]
+    template = "{}{}" if snd_part.startswith("/") else "{}/{}"
+    concatenated = template.format(fst_part, snd_part)
+    return concatenated
+
+
+class MultipartEncoder(object):
+    def __init__(self, fields):
+        mp_encoder = encoder.MultipartEncoder(fields=fields)
+        self.monitor = encoder.MultipartEncoderMonitor(mp_encoder, callback=self._create_callback(mp_encoder))
+
+    def get_monitor(self):
+        return self.monitor
+
+    @staticmethod
+    def _create_callback(encoder_obj):
+        pass
+
+
+class MultipartEncoderWithProgressbar(MultipartEncoder):
+    @staticmethod
+    def _create_callback(encoder_obj):
+        bar = progressbar.ProgressBar(max_value=encoder_obj.len)
+
+        def callback(monitor):
+            if monitor.bytes_read == bar.max_value:
+                bar.finish()
+            else:
+                bar.update(monitor.bytes_read)
+
+        return callback
+
+
+class PathParser(object):
+    LOCAL_DIR = 0
+    LOCAL_FILE = 1
+    GIT_URL = 2
+    S3_URL = 3
+
+    @classmethod
+    def parse_path(cls, path):
+        if cls.is_local_dir(path):
+            return cls.LOCAL_DIR
+
+        if cls.is_local_zip_file(path):
+            return cls.LOCAL_FILE
+
+        if cls.is_git_url(path):
+            return cls.GIT_URL
+
+        if cls.is_s3_url(path):
+            return cls.S3_URL
+
+        raise sdk_exceptions.WrongPathError("Given path is neither local path, nor valid URL")
+
+    @staticmethod
+    def is_local_dir(path):
+        return os.path.exists(path) and os.path.isdir(path)
+
+    @staticmethod
+    def is_local_zip_file(path):
+        return os.path.exists(path) and os.path.isfile(path) and path.endswith(".zip")
+
+    @staticmethod
+    def is_git_url(path):
+        return not os.path.exists(path) and path.endswith(".git") or path.lower().startswith("git:")
+
+    @staticmethod
+    def is_s3_url(path):
+        return not os.path.exists(path) and path.lower().startswith("s3:")
