@@ -1,4 +1,5 @@
 import abc
+import json
 import pydoc
 
 import six
@@ -9,19 +10,16 @@ from halo import halo
 from gradient import api_sdk, exceptions, Job, JobArtifactsDownloader, cli_constants
 from gradient.api_sdk import config, sdk_exceptions
 from gradient.api_sdk.clients import http_client
-from gradient.api_sdk.clients.base_client import BaseClient
+from gradient.api_sdk.clients.base_client import BaseClient, TagsSupportMixin
 from gradient.api_sdk.repositories.jobs import RunJob
-from gradient.api_sdk.utils import print_dict_recursive, urljoin
+from gradient.api_sdk.utils import print_dict_recursive, concatenate_urls, MultipartEncoder
 from gradient.cli_constants import CLI_PS_CLIENT_NAME
 from gradient.cliutils import get_terminal_lines
-from gradient.commands.common import BaseCommand
-from gradient.workspace import MultipartEncoder
+from gradient.commands.common import BaseCommand, StreamMetricsCommand, LogsCommandMixin
 
 
 @six.add_metaclass(abc.ABCMeta)
 class BaseJobCommand(BaseCommand):
-    entity = "job"
-
     def _get_client(self, api_key, logger_):
         client = api_sdk.clients.JobsClient(
             api_key=api_key,
@@ -51,7 +49,7 @@ class BaseCreateJobCommandMixin(object):
 
     @staticmethod
     def get_instance_url(instance_id):
-        url = urljoin(config.config.WEB_URL, "console/jobs/{}".format(instance_id))
+        url = concatenate_urls(config.config.WEB_URL, "console/jobs/{}".format(instance_id))
         return url
 
     def _handle_workspace(self, instance_dict):
@@ -158,40 +156,9 @@ class ListJobsCommand(BaseJobCommand):
         return table_string
 
 
-class JobLogsCommand(BaseJobCommand):
-
-    def execute(self, job_id, line, limit, follow):
-        if follow:
-            self.logger.log("Awaiting logs...")
-            self._log_logs_continuously(job_id, line, limit)
-        else:
-            self._log_table_of_logs(job_id, line, limit)
-
-    def _log_table_of_logs(self, job_id, line, limit):
-        logs = self.client.logs(job_id, line, limit)
-        if not logs:
-            self.logger.log("No logs found")
-            return
-
-        table_str = self._make_table(logs, job_id)
-        if len(table_str.splitlines()) > get_terminal_lines():
-            pydoc.pager(table_str)
-        else:
-            self.logger.log(table_str)
-
-    def _log_logs_continuously(self, job_id, line, limit):
-        logs_gen = self.client.yield_logs(job_id, line, limit)
-        for log in logs_gen:
-            log_msg = "{}\t{}".format(*self._format_row(log))
-            self.logger.log(log_msg)
-
-    @staticmethod
-    def _format_row(log_row):
-        return (style(fg="red", text=str(log_row.line)),
-                log_row.message)
-
-    def _make_table(self, logs, job_id):
-        table_title = "Job %s logs" % job_id
+class JobLogsCommand(LogsCommandMixin, BaseJobCommand):
+    def _make_table(self, logs, id):
+        table_title = "Job %s logs" % id
         table_data = [("LINE", "MESSAGE")]
         table = terminaltables.AsciiTable(table_data, title=table_title)
 
@@ -200,6 +167,15 @@ class JobLogsCommand(BaseJobCommand):
 
         return table.table
 
+    def _get_log_row_string(self, id, log):
+        log_msg = "{}\t{}".format(*self._format_row(log))
+        return log_msg
+
+    @staticmethod
+    def _format_row(log_row):
+        return (style(fg="red", text=str(log_row.line)),
+                log_row.message)
+
 
 class CreateJobCommand(BaseCreateJobCommandMixin, BaseJobCommand):
 
@@ -211,9 +187,7 @@ class CreateJobCommand(BaseCreateJobCommandMixin, BaseJobCommand):
         return self.client.create(data=data, **json_)
 
 
-class JobRunClient(BaseClient):
-    entity = "job"
-
+class JobRunClient(TagsSupportMixin, BaseClient):
     def __init__(self, http_client_, *args, **kwargs):
         super(JobRunClient, self).__init__(*args, **kwargs)
         self.client = http_client_
@@ -281,7 +255,7 @@ class JobRunClient(BaseClient):
         )
         handle = RunJob(self.api_key, self.logger, self.client).create(job, data=data)
         if tags:
-            self.add_tags(entity_id=handle, entity=self.entity, tags=tags)
+            self.add_tags(entity_id=handle, tags=tags)
         return handle
 
 
@@ -385,11 +359,28 @@ class DownloadArtifactsCommand(BaseJobCommand):
 
 class JobAddTagsCommand(BaseJobCommand):
     def execute(self, job_id, *args, **kwargs):
-        self.client.add_tags(job_id, entity=self.entity, **kwargs)
+        self.client.add_tags(job_id, **kwargs)
         self.logger.log("Tags added to job")
 
 
 class JobRemoveTagsCommand(BaseJobCommand):
     def execute(self, job_id, *args, **kwargs):
-        self.client.remove_tags(job_id, entity=self.entity, **kwargs)
+        self.client.remove_tags(job_id, **kwargs)
         self.logger.log("Tags removed from job")
+
+
+class GetJobMetricsCommand(BaseJobCommand):
+    def execute(self, deployment_id, start, end, interval, built_in_metrics, *args, **kwargs):
+        metrics = self.client.get_metrics(
+            deployment_id,
+            start=start,
+            end=end,
+            built_in_metrics=built_in_metrics,
+            interval=interval,
+        )
+        formatted_metrics = json.dumps(metrics, indent=2, sort_keys=True)
+        self.logger.log(formatted_metrics)
+
+
+class StreamJobMetricsCommand(StreamMetricsCommand, BaseJobCommand):
+    pass
